;********************************************************************************************************
;
;	KEYBOARD HID Descriptor - In and Out Reports (2 bytes each).  No Feature Report implemented.
;
;	Allows an AT90USB162 to enumerate as a Keyboard HID device, using the Consumer usage tables.  No
;	custom drivers are required for this device to operate using Windows XP.
;
;	Endpoints are defined as follows:
;
;		Control Endpoint	64 bytes	(EP0_PACKET_SIZE)
;		Endpoint 1			32 bytes 	IN endpoint (device -> host)
;		Endpoint 2			32 bytes	OUT endpoint (host -> device)
;
;	References to the USB2.0 and HID1.11 specifications are provided.  The specifications are downloadable 
;	from http://www.usb.org/developers/docs.
;
;	NOTE: The AVR compiler arranges memory in words.  If a descriptor length is an uneven number
;	of bytes, a "pad byte" must be included explicitly (if not, the compiler will add one and will generate
;	a warning message).  The descriptor length calculation must take the pad byte into consideration.
;	Length calculations are in words and are multiplied by 2 to get the number of bytes.
;
;	To use the Keyboard descriptor, set the IN endpoint to send two bytes whenever your device presses a
;	key.  The two bytes to be sent are defined in the Keyboard Scan Code table below.  Note that sending
;	the keycode combination causes the key to repeat indefinitely (be careful when testing!).  To turn
;	off key repeat, send two bytes as all zeros.  For example, to send the "*" key, send two bytes: 0x02, 0x25.
;   On the next IN transaction send two bytes: 0x00, 0x00 (to turn off key repeat).
;
;	The OUT endpoint will return the status of the Caps Lock, Scroll Lock and Num Lock features (of ANY
;	system keyboard currently connected).
;
;	Version: November 25, 2009
;
;********************************************************************************************************

;	Device_Descriptor (USB Section 9.6.1)
DD:
	.DB 18,0x01					; bLength=18d							bDescriptorType (Device)
	.DW 0x0200					; bcdUSB (USB Rev 2.00)
	.DW 0x0000					; bDeviceClass							bDeviceSubClass 
	.DB 0x00,EP0_PACKET_SIZE	; bDeviceProtocol						bMaxPacketSizeEP0 (default=64 bytes)
	.DW 0x03EB					; idVendor (Atmel)
	.DW 0x21FD					; idProduct (Keyboard)
	.DW 0x0000					; bcdDevice
	.DB 1,2,3,1					; iManufacturer, iProduct, iSerialNumber, bNumConfigurations

;	Configuration Descriptor (USB Section 9.6.3)
;	Interface Descriptor (USB Section 9.6.5)
CD:
	.DB 9,0x02					; bLength=9d							bDescriptorType (Configuration)
	.DB ((CD_End - CD)*2 + (HD_End - HD)*2) - 1, 0; 					wTotalLengthL=CD+ID+HD+ED1+ED2-PadByte, wTotalLengthH
	.DB 0x01,0x01				; bNumInterfaces (1)					bConfigValue
	.DB 0x00,0xC0				; iConfiguration						bmAttributes (b7=1 b6=self-powered b5=RWU not supported)
	.DB 100/2,9					; MaxPower (100mA in 2mA units)			[INTERFACE] Length=9d
	.DB 0x04,0					; bDescriptorType (Interface)			bInterfaceNumber (0)
	.DB 0x00,2					; bAlternateSetting						bNumEndpoints (2)
	.DB 0x03,0x00				; bInterfaceClass (HID)					bInterfaceSubClass
	.DB 0x00,0x00				; bInterfaceProtocol					iInterface
CD_End:

;	HID Descriptor (HID Section 6.2.1)
;	Endpoint Descriptors (USB Section 9.6.6)
HD:
	.DB 9,0x21					; bLength=9d							bDescriptorType (HID)
	.DW 0x0111					; bcdHID (HID Rev 1.11)
	.DB 0x00,1					; bCountryCode (none)					bNumDescriptors (1)
	.DB 0x22,((RD_End - RD)*2)-1; bDescriptorType (Report)				wReportDescriptorLengthL

	.DB 0x00,7					; wReportDescriptorLengthH				[ENDPOINT1] bLength=7d
	.DB 0x05,0x81				; bDescriptorType (Endpoint)			bEndpointAddress (IN EP1=0x80|0x01)	
	.DB 0x03,0x20				; bmAttributes (Interrupt)				wMaxPacketSizeL (32 bytes)
	.DB 0x00,0x0A				; wMaxPacketSizeH						bInterval (10 msec)

	.DB 7,0x05					; [ENDPOINT2] bLength=7d				bDescriptorType (Endpoint)	
	.DB 0x02,0x03				; bEndpointAddress (OUT EP2=0x00|0x02)	bmAttributes (Interrupt)
	.DB 0x20,0x00				; wMaxPacketSizeL (32 bytes)			wMaxPacketSizeH
	.DB 0x0A,0x00				; bInterval (10 msec)					*Pad Byte*
HD_End:

;
;	Report Descriptor (HID Section 6.2.2)
;	Simple keyboard descriptor
;
RD:
	.DB	0x05,0x01				; Usage Page (Generic Desktop)			05 01
    .DB	0x09,0x06 				; Usage (Keyboard)						09 06
    .DB	0xA1,0x01				; Collection (Application)				A1 01

;	Input Report:
    .DB	0x05,0x07				; 	Usage Page (Keyboard)				05 07
    .DB	0x19,0xE0				; 	Usage Minimum (Keyboard LeftControl)19 E0
    .DB	0x29,0xE7				; 	Usage Maximum (Keyboard RightGUI)	29 E7
    .DB	0x15,0x00				; 	Logical Minimum (0)					15 00
    .DB	0x25,0x01				; 	Logical Maximum (1)					25 01
    .DB	0x75,0x01				; 	Report Size (1)						75 01
    .DB	0x95,0x08				; 	Report Count (8)					95 08
    .DB	0x81,0x02				; 	Input (Data,Variable,Absolute)		81 02
    .DB	0x95,0x01				; 	Report Count (1)					95 01
    .DB	0x75,0x08				; 	Report Size (8)						75 08
    .DB	0x25,0x65				; 	Logical Maximum (101)				25 65
    .DB	0x19,0x00				; 	Usage Minimum (Reserved)			19 00
    .DB	0x29,0x65				; 	Usage Maximum (Keyboard Application)29 65
    .DB	0x81,0x00				; 	Input (Data, Arbitrary, Absolute)	81 00

;	Output Report:
	.DB 0x09,0x01				; 	Usage (Vendor)						09 01
	.DB 0x15,0x00				; 	Logical Minimum (-127)				15 00
	.DB	0x25,0x01				;	Logical Maximum (128)				25 01
	.DB	0x75,0x08				;	Report Size (8)						75 08
	.DB	0x95,0x02				; 	Report Count (2)					95 02
	.DB	0x91,0x02				;	Output (Data, Var, Abs)				91 02

    .DB	0xC0, 0x00				; End Collection, *Pad Byte*			C0 00
RD_END:

; 	String Descriptors (USB Section 9.6.7).  All strings are 2 byte Unicode

;	Language:
SD0:
	.DB (SD0_End - SD0)*2, 0x03		; bLength		bDescriptorType (string)
	.DW 0x0409						; wLANGID (US English - no other types work)	
SD0_End:	

;	Manufacturer ID:
SD1:
	.DB (SD1_End - SD1)*2, 0x03		; bLength		bDescriptorType (string)
	.DW 'G','E','N','E','R','I','C'
SD1_End:

;	Product ID - displays in the Windows "bubble" when the driver installs:
SD2:
	.DB (SD2_End - SD2)*2, 0x03		; bLength		bDescriptorType (string)
	.DW 'A','V','R','U','S','B','1','6','2','-','K','E','Y','B','O','A','R','D'
SD2_End:

;	Serial Number (Device Instance ID):
SD3:
	.DB (SD3_End - SD3)*2, 0x03		; bLength		bDescriptorType (string)
	.DW '0','0','0','1'
SD3_End:

;********************************************************************************************************
;	Keyboard Scan Codes - partial list
;
;	The first byte = 0x00 (lower case), 0x02 (upper case); the second byte is from table below.
;	When a key is sent, it will repeat until zeros are sent for the First and Second bytes.
;	E.g. Sending 0x0225 will cause the "*" character to be repeated until 0x0000 is sent.
;********************************************************************************************************

;	04	Keyboard a and A
;	05	Keyboard b and B
;	06	Keyboard c and C
;	07	Keyboard d and D
;	08	Keyboard e and E
;	09	Keyboard f and F
;	0A	Keyboard g and G
;	0B	Keyboard h and H
;	0C	Keyboard i and I
;	0D	Keyboard j and J
;	0E	Keyboard k and K
;	0F	Keyboard l and L
;	10	Keyboard m and M
;	11	Keyboard n and N
;	12	Keyboard o and O
;	13	Keyboard p and P
;	14	Keyboard q and Q
;	15	Keyboard r and R
;	16	Keyboard s and S
;	17	Keyboard t and T
;	18	Keyboard u and U
;	19	Keyboard v and V
;	1A	Keyboard w and W
;	1B	Keyboard x and X
;	1C	Keyboard y and Y
;	1D	Keyboard z and Z
;	1E	Keyboard 1 and !
;	1F	Keyboard 2 and @
;	20	Keyboard 3 and #
;	21	Keyboard 4 and $
;	22	Keyboard 5 and %
;	23	Keyboard 6 and ^
;	24	Keyboard 7 and &
;	25	Keyboard 8 and *
;	26	Keyboard 9 and (
;	27	Keyboard 0 and )



