;******************************************************************************
;
;	Standard Macros
;
;	Version: November 24, 2009
;
;******************************************************************************
; Minimum - Find the minimum between two registers and return the minimum in Rd
.macro Min								; Min rd,rr
	cp		@0,@1
	brlo	_ex
	mov		@0,@1
_ex:nop
.endm

; Minimum Word - Find the minimum between two register pairs and return the minimum in RdH:RdL
.macro Minw								; Min rdH, rdL ,rrH, rrL
	cp		@1,@3
	cpc		@0,@2
	brlo	_ex
	mov		@0,@2
	mov		@1,@3
_ex:nop
.endm

; Load Immediate Word
.macro	Ldiw							; Ldiw Z,k - Load Z with word k
	ldi		@0H,HIGH(@1)	
	ldi	 	@0L,LOW(@1)	
.endm

; Add Word
.macro	Addw							; Addw Z,reg - Add r to Z
	clr	r0								; assumes lpm instruction will follow macro
	add	@0L,@1
	adc	@0H,r0
.endm

; Set Flag register bit
.macro	SetFLAG							; SetFLAG b, b={0..7}
	sbi		GPIOR0,@0
.endm

; Clear Flag register bit
.macro	ClrFLAG							; ClrFLAG b, b={0..7}
	cbi		GPIOR0,@0
.endm

; Wait
.macro	Wait							; Wait - Wait here (debug)
	rjmp	PC
.endm

; CPU power down macros
.macro	ConfigureSLEEPMode
	ldi		temp,(1<<SM1)				; Configure CPU 'Power Down' mode
	out		SMCR,temp
.endm

.macro	EnableSLEEP
	in		temp,SMCR
	sbr		temp,(1<<SE)
	out		SMCR,temp
.endm

.macro	DisableSLEEP
	in		temp,SMCR
	cbr		temp,(1<<SE)
	out		SMCR,temp
.endm

.macro	DisableCOMPARATOR
	ldi		temp,(1<<ACD)
	out		ACSR,temp
.endm

.macro	EnabelCOMPARATOR
	ldi		temp,(0<<ACD)
	out		ACSR,temp
.endm

;	~5ms Delay at 16MHz
.macro	Delay_5ms
	push	r16
	push	r17
	ldi		r17,0x70
_l2:ldi		r16,0xFF
_l1:dec		r16
	brne	_l1
	dec		r17
	brne	_l2
	pop		r17
	pop		r16
.endm


